@@ -185,7 +185,7 @@ void print_hash(const struct isds_hash *hash) {
 
     if (!hash->value) printf("<NULL>");
     else
-        for (int i = 0; i < hash->length; i++) {
+        for (size_t i = 0; i < hash->length; i++) {
             if (i > 0) printf(":");
             printf("%02x", ((uint8_t *)(hash->value))[i]);
         }
@@ -387,7 +387,7 @@ void print_timeval(const struct timeval *time) {
     if (!localtime_r(&(time->tv_sec), &broken)) goto error;
     if (!strftime(buffer, sizeof(buffer)/sizeof(char), "%c", &broken))
         goto error;
-    printf("%s, %ld us\n", buffer, time->tv_usec);
+    printf("%s, %jd us\n", buffer, (intmax_t)time->tv_usec);
     return;
 
 error:
