@@ -8,7 +8,7 @@
 #include "common.h"
 
 
-int main(int argc, char **argv) {
+int main(void) {
     isds_error err;
     
     setlocale(LC_ALL, "");
