@@ -1,5 +1,6 @@
 #ifndef _XOPEN_SOURCE
-#define _XOPEN_SOURCE 500   /* For strdup() */
+#include "../../config.h"
+#define _XOPEN_SOURCE XOPEN_SOURCE_LEVEL_FOR_STRDUP
 #endif
 
 #include "http.h"
@@ -587,7 +588,7 @@ static int find_content_length(struct http_request *request) {
             return HTTP_ERROR_SERVER;
         if (value < 0)
             return HTTP_ERROR_CLIENT;
-        if (value > SIZE_MAX)
+        if ((unsigned long long)value > SIZE_MAX)
             return HTTP_ERROR_SERVER;
         request->body_length = value;
     } else {
